package td;

/**
 * Une liste simplement chaine, une cellule vide reprsente la tte
 * @author jub
 * @param <X> Le type des lements de la liste
 */
public class Liste<X> {
	/**
	 * La liste est implmente en utilisant une cellule la tte
	 */
	private Cellule<X> tete;
	/**
	 * Construit une liste vide compose d'une unique cellule la tte
	 */
	public Liste()
	{
		this.tete = new Cellule();
	}

	/**
	 * @return vrai si la liste est vide, faux sinon
	 */
	public boolean estVide()
	{
		return this.tete.getSuivant()==null;
	}

	/**
	 * @return Le dernier lment de la liste
	 */
	public X getDernier()
	{
		X res = null;
		Cellule pos = tete;
		while (pos.getSuivant()!=null)
		{
			pos = pos.getSuivant();
		}
		res = (X) pos.getValeur();
		return res;
	}

	/**
	 * @return le premier lment de la liste (la tte n'est pas prise en compte)
	 */
	public X getPremier()
	{
		X res =null;
		if (!this.estVide())
		{
			res = tete.getSuivant().getValeur();
		}
		return res;
	}

	/**
	 * @return le nombre d'lments de la liste
	 */
	public int getTaille()
	{
		int i =0;
		Cellule pos = tete;
		while (pos.getSuivant()!=null)
		{
			pos = pos.getSuivant();
			i++;
		}
		return i;
	}

	/**
	 * Renvoie la valeur de l'lment qui correspond  l'indice.
	 * Le premier lment  l'indice 1  
	 * @param indice l'indice de l'lment auquel on souhaite accder.
	 * @return l'lment selectionn si sa position existe, null sinon.
	 */
	public X getVal(int indice)
	{
		int i =0;
		X res;
		Cellule pos = tete;
		while (pos.getSuivant()!=null && i < indice)
		{
			pos = pos.getSuivant();
			i++;
		}
		if (i==indice)
		{
		res= (X) pos.getValeur();
		}
		else
		{
			res = null;
		}
		return res;
	}

	/**
	 * Insert val  la position indice
	 * Le premier lment a pour indice 1;
	 * @param val la valeur a inserer
	 * @param indice la position ou inserer
	 */
	public void insertPos(X val, int indice)
	{
		int i =1;
		X res;
		Cellule pos = tete;
		while (pos.getSuivant()!=null && i < indice)
		{
			pos = pos.getSuivant();
			i++;
		}
		Cellule<X> t = new Cellule<X>();
		t.setValeur(val);
		t.setSuivant(pos.getSuivant());
		pos.setSuivant(t);
	}

	/**
	 * Insert un lment en queue
	 * @param val la valeur  inserer
	 */
	public void insertQueue(X val)
	{
		Cellule<X> t = new Cellule<X>();
		t.setValeur(val);
		t.setSuivant(null);
		Cellule pos = tete;
		while (pos.getSuivant()!=null)
		{
			pos = pos.getSuivant();
		}
		pos.setSuivant(t);

	}
	
	/**
	 * Insert un lmnt en tte
	 * @param val la valeur  inserer
	 */
	public void insertTete(X val)
	{
		Cellule<X> t = new Cellule<X>();
		t.setValeur(val);
		t.setSuivant(tete.getSuivant());
		tete.setSuivant(t);

	}
	
	/**
	 * Modifie la valeur d'un lment donn par son indice
	 * Le premier lment est  la position 1
	 * @param val la nouvelle valeur
	 * @param indice la position
	 */
	public void setVal(X val, int indice)
	{
		int i =0;
		X res;
		Cellule pos = tete;
		while (pos.getSuivant()!=null && i < indice)
		{
			pos = pos.getSuivant();
			i++;
		}
		pos.setValeur(val);
	}
	
	/**
	 * Supprime un lmnt  la position donne.
	 * Le premier lment est en position 1
	 * @param indice la position de l'lmnt  supprimer
	 */
	public void suppressionPosition(int indice)
	{
		if (!this.estVide())
		{
			int i = 1;
			Cellule pos = tete;
			while (pos.getSuivant().getSuivant()!=null && i < indice)
			{
				pos = pos.getSuivant();
				i++;
			}
			pos.setSuivant(pos.getSuivant().getSuivant());
		}
	}
	
	/**
	 * Supprime le dernier lment
	 */
	public void suppressionQueue()
	{
		if (!this.estVide())
		{
			Cellule pos = tete;
			while (pos.getSuivant().getSuivant()!=null)
			{
				pos = pos.getSuivant();
			}
			pos.setSuivant(pos.getSuivant().getSuivant());
		}
	}
	
	/**
	 * Supprime le premier lment
	 */
	public void suppressionTete()
	{
		if (!this.estVide())
			this.tete.setSuivant(this.tete.getSuivant().getSuivant());
	}
	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return tete.toString();
	}
}
