package td;

/**
 * Une classe utilitaire pour manipuler des tableaux d'entiers
 * @author jub
 */
public class TabInt {
	/**
	 * @param t le tableau  afficher
	 */
	public static void affiche(int [] t)
	{	
		for (int i=0; i<t.length; i++)
			System.out.println("[" +i+"]="+t[i]);
	}

	/**
	 * Permet d'extraire un sous tableau
	 * @param t le tableau source
	 * @param debut le debut de la plage
	 * @param fin la fin de la plage
	 * @return le tableau extrait de t en utilisant la plage dfinie par debut et fin
	 */
	public static int[] extraction(int[] t, int debut, int fin)
	{
		int[] res;
		if (0<=fin-debut && fin-debut < t.length)
		{
			res = new int[fin-debut+1];
			for (int i=0; i < res.length; i++)
			{
				res[i]=t[debut+i];
			}
		}
		else
		{
			res = null;
		}
		return res;
	}

	/**
	 * Ralise la fusion de deux tableaux
	 * @param t1
	 * @param t2
	 * @return le tableau resultant de la fusion 
	 */
	public static int[] fussion(int[] t1, int[] t2)
	{
		int [] res = new int[t1.length + t2.length];
		for (int i=0; i < t1.length; i++)
		{
			res[i]=t1[i];
		}
		for (int i=0; i < t2.length; i++)
		{
			res[i+t1.length]=t2[i];
		}
		return res;
	}

	/**
	 * Insert un lment dans un tableau, le dernier lment est perdu
	 * @param t le tableau o l'lment sera inser 
	 * @param elt l'lment  inserer
	 * @param pos la position de l'insertion
	 */
	public static void insertion(int[] t, int elt, int pos)
	{
		for (int i = t.length - 1; i > pos ;  i-- )
		{
			t[i]=t[i-1];
		}
		t[pos]=elt;
	}

	/**
	 * @param t le tableau qui sera invers
	 */
	public static void inversion(int[] t)
	{
		int temp;
		for(int i=0; i<t.length/2; i++)
		{
			temp = t[t.length-1-i];
			t[t.length-1-i]=t[i];
			t[i]=temp;
		}
	}

	/**
	 * @param t le tableau dont on cherche le maximum
	 * @return la valeur du maximum
	 */
	public static int max(int[] t)
	{
		return t[TabInt.posMax(t)];
	}

	/**
	 * Version iterative de la recherche de la position du maximum
	 * @param t le tableau dont on cherche la position du maximum
	 * @return la position du maximum
	 */
	public static int posMax(int[] t)
	{
		int pos =0; 
		for (int i=1; i<t.length; i++)
		{
			if (t[pos]<t[i])
			{
				pos=i;
			}
		}
		return pos;
	}

	/**
	 * Version recursive de la recherche de la position du maximum
	 * @param t tableau dont on cherche la position du maximum
	 * @param pos l'indice qui dfinit la plage de recherche
	 * @return la position du maximum
	 */
	public static int posMaxR(int[] t, int pos)
	{
		int m = 0;
		int res;
		if (pos == t.length -1)
		{
			res = pos;
		}
		else
		{
			m = posMaxR(t, pos+1);

			if (t[pos] >= t[m])
			{
				res = pos;
			}
			else
			{
				res = m;
			}
		}	
		return res;
	}




	/**
	 * Recherche dichotomique dans un tbleau tri
	 * @param t le tableau tri o la recherche est effectue
	 * @param elt l'lment recherch
	 * @return la position de la premire occurence de l'lment recherch et -1 sinon
	 */
	public static int rechercheDicoTrie(int[] t, int elt)
	{
		int inf =0;
		int sup = t.length-1;
		int milieu;
		int position = -1;
		boolean trouve = false;


		while (!trouve && inf <= sup)
		{
			milieu = (inf + sup)/2;
			if (t[milieu]==elt)
			{
				trouve = true;
				position = milieu;
			}
			else
				if (t[milieu]>elt)
				{
					sup = milieu -1;
				}
				else
				{
					inf = milieu +1;
				}
		}
		return position;
	}


	/**
	 * Recherche squentielle dans un tableau non tri
	 * @param t be tableau o la recherche est effectue
	 * @param elt l'lment recherch
	 * @return la position de la premire occurence de l'lment recherch et -1 sinon
	 */
	public static int rechercheSeqNonTrie(int[] t, int elt)
	{
		int i = 0;
		int position = -1;
		boolean trouve = false;

		while (!trouve && i <t.length)
		{
			if (t[i] ==elt)
			{
				trouve = true;
				position=i;
			}
			else
			{
				i = i + 1;
			}
		}
		return position;

	}

	/**
	 * Recherhce squentielle dans un tableau tri
	 * @param t be tableau tri o la recherche est effectue
	 * @param elt l'lment recherch
	 * @return la position de la premire occurence de l'lment recherch et -1 sinon@param t
	 */
	public static int rechercheSeqTrie(int[] t, int elt)
	{
		int i = 0;
		int position = -1;
		boolean arret = false;

		while (!arret && i <t.length)
		{
			if (t[i] ==elt)
			{
				arret = true;
				position=i;
			}
			else
			{
				if (t[i]>elt)
				{
					arret = true;
				}
				else
				{	
					i = i + 1;
				}	
			}
		}
		return position;
	}

	/**
	 * Rempli alatoirement un tableau
	 * @param t le tableau qui sera rempli
	 * @param max la valeur maximum
	 */
	public static void remplissageA(int [] t,int max)
	{
		for (int i=0; i<t.length; i++)
			t[i]=(int) (Math.random()*max);
	}

	/**
	 * Supprime un lment  une position donne
	 * Le dernier lment n'est plus significatif 
	 * @param t le tableau
	 * @param pos la position de l'lment  supprimer 
	 */
	public static void suppression(int[] t, int pos)
	{
		for (int i = pos; i<t.length -1; i++)
		{
			t[i]=t[i+1];
		}

	}
	/**
	 * Tri  bulle
	 * @param t le tableau  trier
	 */
	public static void triBulle(int[] t)
	{

		boolean permut;
		int tmp;

		do
		{
			//hypothse : le tableau est tri
			permut=false;
			for(int i=0;i<t.length-1;i++)
			{
				//Teste si 2 lments successifs sont dans le bon ordre ou non
				if(t[i]>t[i+1])
				{
					//s'ils ne le sont pas on change leurs positions
					tmp = t[i];
					t[i] = t[i+1];
					t[i+1]=tmp;
					permut=true;
				}
			}
		}
		while(permut);
	}

	/**
	 * Tri par insertion
	 * @param t le tableau  trier
	 */
	public static void triInsertion(int[] t)
	{
		boolean bRetour = false;
		int cpt;
		int i;
		int lg;
		int val;
		lg = t.length;

		for(i = 1; i < lg ; i++)
		{    
			val = t[i];
			cpt = i-1;
			do
			{
				bRetour = true;
				if(t[cpt] > val)
				{    
					t[cpt+1] = t[cpt];
					cpt = cpt-1;
					bRetour = false;
				}
				if(cpt <0)
				{
					bRetour = true;
				}
			}while(!bRetour);
			t[cpt+1] = val;
		}

	}

	/**
	 * Tri par selection
	 * @param t le tableau  trier
	 */
	public static void triSelection(int[] t)
	{
		int i, min, j , m;
		for(i = 0 ; i < t.length - 1 ; i++)
		{
			min = i;
			for(j = i+1 ; j < t.length ; j++)
				if(t[j] < t[min])
					min = j;
			if(min != i)
			{
				m = t[i];
				t[i] = t[min];
				t[min] = m;
			}
		}
	}

	/**
	 * Calcul itratif la somme des lments d'un tableau
	 * @param t le tableau dont la somme est calcule
	 * @return la somme des lments
	 */
	public static int somme(int[] t)
	{
		int res = 0;
		for (int i=0; i < t.length; i++)
		{
			res = res + t[i];
		}
		return res;
	}

	/**
	 * Calcul recursif de la somme des lments d'un tableau
	 * @param t le tableau dont la somme est calcule
	 * @return la somme des lments
	 */
	public static int sommeR(int[] t)
	{
		int res = 0;
		if (t.length ==1)
		{
			res = t[0];
		}
		else
		{	
			res = t[0]+sommeR(extraction(t, 1, t.length-1));
		}
		return res;
	}

	
	/**
	 * Calcul recursif de la somme des lments d'un tableau
	 * @param t le tableau dont la somme est calcule
	 * @param pos dfini le sous tableau non encore tri 
	 * @return la somme des lments
	 */
	public static int sommeR2(int[] t, int pos)
	{
		int res;
		if (pos == t.length -1)
		{
			res = t[pos];
		}
		else
		{
			res = t[pos] + sommeR2(t, pos +1);
		}
		return res;
	}
}

